; -------------------------------------------------------------------------
;                     	        MG Macro List
; -------------------------------------------------------------------------
; Written: April 14, 2006
; By: Mario Gutierrez
; Version: 2.00
; Revised: May 17, 2011
;
; -------------------------------------------------------------------------
; To use this macros, they must be included in the header of the program
; as shown below. (also see below for using word macros)
;	#include <MG Macros.inc>	; List of Macro definitions
;
; For use with Microchip MPLab MPASM.
; Microchip & MPLab are register trade marks of Microchip Technology Inc.
;
; When using values in registers, just add an f to the end of the instru-
; tion, all except mov, use movx because movf is already used by Mpasm.
; The other exception is movbx = move NOT bit.
; example:  add = for literal value, addf = for value in register.
; Use lower case z & c, Upper case only points to a bit number.
;
; Word Macros, working with two registers.
;
; To use the word macros, the words must be defined in the program.
; Defining a word:
; #define	volts	volts_H, volts_L
; #define	timer	timer_H, timer_L
;
; NOTE: It is very important to always remember that their is a difference
; between working with registers and literal values, the little "f" makes
; a huge difference.
; A word is composed of frH and frL.
;
; =========================================================================
;                     	    Standard Macro List
; =========================================================================
; -------------------------------------------------------------------------
; Mathematics Functions.
; -------------------------------------------------------------------------
; add		; Adds a literal value to register fr1.
; addf		; Adds two registers together and puts the result in fr1.
; sub		; Substracts a literal value from register fr1.
; subf		; Substracts register fr2 from register fr1, result in fr1.
; inc		; Increment register
; incw		; Increment W.
; dec		; Decrement register
; decw		; Decrement W.
; mult		; Multiplies fr1 x W, results go to frLo & frHi.
; div		; Divide fr1 by fr2, result goes to result, remainder in fr1.

; -------------------------------------------------------------------------
; Comparason Functions.
; -------------------------------------------------------------------------
; cja		; Compare register jump if above literal value.
; cjaf		; Compare two registers jump if above.
; cjae		; Compare register jump if above or equal to literal value.
; cjaef		; Compare two registers jump if above or equal.
; cjb		; Compare register jump if below literal value.
; cjbf		; Compare two registers jump if below literal value.
; cjbe		; Compare register jump if below or equal to literal value.
; cjbef		; Compare two registers jump if below or equal.
; cje		; Compare register jump if equal to literal value.
; cjef		; Compare two registers jump if equal.
; cjne		; Compare register jump if not equal to literal value.
; cjnef		; Compare two registers and jump if not equal.
; csa		; Compare register skip if above literal value.
; csaf		; Compare two registers skip if above.
; csae		; Compare register skip if above or equal literal value.
; csaef		; Compare two registers skip if above or equal.
; csb		; Compare register skip if below the literal value.
; csbf		; Compare two registers skip if fr1 is below fr2.
; csbe		; Compare register skip if below or equal literal value.
; csbef		; Compare two registers skip if fr1 is below or equal to fr2.
; cse		; Compare register skip if equal to literal value.
; csef		; Compare two registers skip if equal.
; csne		; Compare register skip if not equal to literal value.
; csnef		; Compare two registers skip if not equal.

; -------------------------------------------------------------------------
; Logical Functions.
; -------------------------------------------------------------------------
; and		; AND register with a literal value.
; andf		; AND register fr1 with value in register fr2.
; or		; OR registers fr1 with a literal value.
; orf		; OR register fr1 with value in register fr2.
; xor		; XOR register fr1 with a literal value.
; xorf		; XOR registers fr1 with value in register fr2.

; -------------------------------------------------------------------------
; Other Functions.
; -------------------------------------------------------------------------
; clr		; Clear register.
; jmp		; Jump to label.
; sz		; Skip if Zero.
; snz		; Skip if Not Zero. 
; sc		; Skip if Carry.
; snc		; Skip if No Carry.
; skip		; Skip following step.
; test		; Test register for zero.
; testw		; Test value, affects Z, DC and C.
; rl		; Rotate left fr.
; rr		; Rotate right fr.
; jc		; Jump to label if C is set.
; jnc		; Jump to label if C is cleared.
; jz		; Test Z, Jump to label if zero.
; jnz		; Test Z, Jump to label if not zero.
; ijnz		; Increment register jump if not zero.
; djnz		; Decrement register jump if not zero.
; mov		; Copy Literal value to register fr1.
; movx		; Copy register fr2 to register fr1.
; invw		; Exchanges the contents of W with register fr1.
; invf		; Exchanges the contents between two registers.
; swap		; Inverts fr1's Upper Nibble with Lower Nibble.

; -------------------------------------------------------------------------
; Bit Control function.
; -------------------------------------------------------------------------
; clrb		; Clear bit.
; setb		; Set bit.
; sb		; Skip next instruction if bit = 1.
; snb		; Skip next instruction if no bit.
; addb		; Add bit "fr2,bit" to register fr1
; subb		; Subtract bit "fr2,bit" from register fr1
; jb		; Jump to label if bit.
; jnb		; Jump to label if no bit.
; movb		; Move bit.
; movbx		; Move NOT bit.
; xorb		; XOR bit, flip bit.

; -------------------------------------------------------------------------
; Other Control function.
; -------------------------------------------------------------------------
; INDIRECT	; Access fr through FSR.
; bank0		; Move to Bank0.
; bank1		; Move to Bank1.
; bank2		; Move to Bank2.
; bank3		; Move to Bank3.

; =========================================================================
;                     	      Word Macro List
; =========================================================================
; -------------------------------------------------------------------------
; Mathematics Functions.
; -------------------------------------------------------------------------
; wadd		; Adds a 2-byte literal value to word.
; waddf		; Adds two words together and puts the result in wd1.
; wsub		; Substracts a 2-byte literal value from word wd1.
; wsubf		; Substracts word wd2 from word wd1, result in wd1.
; winc		; Increment word.
; wdec		; Decrement word.
; wdiv		; Divide word wd1(fr1H/L) by 2-byte literal value
; wdivf		; Divide words fr1 by fr2, result goes to RsltH/L
; wmult		; Multiplies fr1 x W, result goes to Result_H/L.

; -------------------------------------------------------------------------
; Comparason Functions.
; -------------------------------------------------------------------------
; wcja		; Compare word jump if above 2-byte literal value.
; wcjaf		; Compare two words jump if wd1 is above wd2.
; wcjae		; Compare word jump if above or equal to 2-byte literal value.
; wcjaef	; Compare two words jump if wd1 is above or equal to wd2.
; wcjb		; Compare word jump if below 2-byte literal value.	
; wcjbf		; Compare two words jump if wd1 is below wd2.
; wcjbe		; Compare word jump if below or equal to 2-byte literal value.
; wcjbef	; Compare two words jump if wd1 is below or equal to wd2.
; wcje		; Compare word jump if equal to 2-byte literal value.
; wcjef		; Compare two words jump if equal.
; wcjne		; Compare word jump if not equal to 2-byte literal value.
; wcjnef	; Compare two words and jump if not equal.
; wcsa		; Compare word skip if above 2-byte literal value.
; wcsaf		; Compare two wordss skip if wd1 is above wd2.
; wcsae		; Compare word skip if above or equal 2-byte literal value.
; wcsaef	; Compare two words skip if wd1 is above or equal to wd2.
; wcsb		; Compare word skip if below the 2-byte literal value.
; wcsbf		; Compare two words skip if wd1 is below wd2.
; wcsbe		; Compare word skip if below or equal 2-byte literal value.
; wcsbef	; Compare two words skip if wd1 is below or equal to wd2.
; wcse		; Compare word skip if equal to 2-byte literal value.
; wcsef		; Compare two words skip if equal.
; wcsne		; Compare word skip if not equal to 2-byte literal value.
; wcsnef	; Compare two words skip if not equal.

; -------------------------------------------------------------------------
; Logical Functions.
; -------------------------------------------------------------------------
; wand		; AND word with a 2-byte literal value.
; wandf		; AND word wd1 with value in word wd2.
; wor		; OR word wd1 with a 2-byte literal value.
; worf		; OR word wd1 with value in word wd2.
; wxor		; XOR word wd1 with a 2-byte literal value.
; wxorf		; XOR word wd1 with value in word wd2.

; -------------------------------------------------------------------------
; Other Functions.
; -------------------------------------------------------------------------
; wclr		; Clears the contents in word.
; wtest		; Check word Z status.
; wrl		; Rotate word bits left wd1.
; wrr		; Rotate word bits right wd1.
; wdjnz		; Decrement word jump if not zero, try using wijnz.
; wijnz		; Increment word jump if not zero, more efficient than wdjnz.
; wmov		; Copy 2-byte Literal value to word wd1.
; wmovx		; Copy word wd2 to word wd1.
; wswap		; Swap the contents between H and L bytes of word wd1.
;

; =========================================================================
; ********** Begining of standard macros
; =========================================================================
#define c STATUS,0
#define z STATUS,2
#define INDIRECT INDF

; -------------------------------------------------------------------------
; Mathematics Functions.
; -------------------------------------------------------------------------
; Adds a literal value to register fr1.
add	MACRO	fr1,lit
	movlw	lit
	addwf	fr1,f
	endm

; Adds two registers together and puts the result in fr1.
addf MACRO	fr1,fr2
	movf	fr2,w
	addwf	fr1,f
	endm

; Substracts a literal value from register fr1.
sub MACRO	fr1,lit
	movlw	lit
	subwf	fr1,f
	endm

; Substracts register fr2 from register fr1, result in fr1.
subf MACRO	fr1,fr2
	movf	fr2,w
	subwf	fr1,f
	endm

; Increment register
inc MACRO	fr1
	incf	fr1,f
	endm

; Increment W.
incw MACRO
	addlw	0x01
	endm

; Decrement register
dec MACRO	fr1
	decf	fr1,f
	endm

; Decrement W.
decw MACRO
	addlw	0xFF
	endm

; Multiplies fr1 x W, results go to frLo & frHi.
; Hi byte = frHi, Lo byte = frLo.     
; value in fr1 will is destroyed.     
mult MACRO	frHi,frLo,fr1;
	LOCAL loop
	clrf	frLo
	clrf	frHi
	bsf		frLo,7		; Set bit 7 as loop counter.
loop
	rrf		fr1,f
	skpnc
	addwf	frHi,f		; Add W to fr4 if Carry.
	rrf		frHi,f		; Rotate 16 bit result.
	rrf		frLo,f
	skpc				; End loop if Carry (after 8 times)
	goto	loop
	endm

; Divide fr1 by fr2, result goes to result, remainder in fr1.
; If fr1 is less than fr2, the result will be zero and fr1 will remain.
; value in fr1 will change to remainder.     
div MACRO	Result,fr1,fr2
	LOCAL loop,exit
	clrf	Result
loop
	movf	fr2,w			; Exit if fr2 is or reached zero.
	skpnz
	goto	exit
	subwf	fr1,w			; Compare, exit if fr1 is greater.
	btfss	STATUS,0
	goto	exit
	movf	fr2,w			; Subtract one count from dividend.
	subwf	fr1,f
	incf	Result			; Add one to result for every subtraction.
	goto	loop
exit
	endm

; -------------------------------------------------------------------------
; Comparason Functions.
; -------------------------------------------------------------------------
; Compare register jump if above literal value.
cja MACRO	fr1,Lit,Label
	movlw	Lit ^ 0xFF
	addwf	fr1,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare two registers jump if above.
cjaf MACRO	fr1,fr2,Label
	movf	fr1,w
	subwf	fr2,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare register jump if above or equal to literal value.
cjae MACRO	fr1,Lit,Label
	movlw	Lit
	subwf	fr1,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare two registers jump if above or equal.
cjaef MACRO	fr1,fr2,Label
	movf	fr2,w
	subwf	fr1,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare register jump if below literal value.
cjb MACRO	fr1,Lit,Label
	movlw	Lit
	subwf	fr1,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare two registers jump if below literal value.
cjbf MACRO	fr1,fr2,Label
	movf	fr2,w
	subwf	fr1,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare register jump if below or equal to literal value.
cjbe MACRO	fr1,Lit,Label
	movlw	Lit ^ 0xFF
	addwf	fr1,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare two registers jump if below or equal.
cjbef MACRO	fr1,fr2,Label
	movf	fr1,w
	subwf	fr2,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare register jump if equal to literal value.
cje MACRO	fr1,Lit,Label
	movlw	Lit
	subwf	fr1,w
	btfsc	STATUS,2
	goto	Label
	endm

; Compare two registers jump if equal.
cjef MACRO	fr1,fr2,Label
	movf	fr2,w
	subwf	fr1,w
	btfsc	STATUS,2
	goto	Label
	endm

; Compare register jump if not equal to literal value.
cjne MACRO	fr1,Lit,Label
	movlw	Lit
	subwf	fr1,w
	btfss	STATUS,2
	goto	Label
	endm

; Compare two registers and jump if not equal.
cjnef MACRO	fr1,fr2,Label
	movf	fr2,w
	subwf	fr1,w
	btfss	STATUS,2
	goto	Label
	endm

; Compare register skip if above literal value.
csa MACRO	fr1,Lit
	movlw	Lit ^ 0xFF
	addwf	fr1,w
	btfss	STATUS,0
	endm

; Compare two registers skip if above.
csaf MACRO	fr1,fr2
	movf	fr1,w
	subwf	fr2,w
	btfsc	STATUS,0
	endm

; Compare register skip if above or equal literal value.
csae MACRO	fr1,Lit
	movlw	Lit
	subwf	fr1,w
	btfss	STATUS,0
	endm

; Compare two registers skip if above or equal.
csaef MACRO	fr1,fr2
	movf	fr2,w
	subwf	fr1,w
	btfss	STATUS,0
	endm

; Compare register skip if below the literal value.
csb MACRO	fr1,Lit
	movlw	Lit
	subwf	fr1,w
	btfsc	STATUS,0
	endm

; Compare two registers skip if fr1 is below fr2.
csbf MACRO	fr1,fr2
	movf	fr2,w
	subwf	fr1,w
	btfsc	STATUS,0
	endm

; Compare register skip if below or equal literal value.
csbe MACRO	fr1,Lit
	movlw	Lit ^ 0xFF
	addwf	fr1,w
	btfsc	STATUS,0
	endm

; Compare two registers skip if fr1 is below or equal to fr2.
csbef MACRO	fr1,fr2
	movf	fr1,w
	subwf	fr2,w
	btfss	STATUS,0
	endm

; Compare register skip if equal to literal value.
cse MACRO	fr1,Lit
	movlw	Lit
	subwf	fr1,w
	btfss	STATUS,2
	endm

; Compare two registers skip if equal.
csef MACRO	fr1,fr2
	movf	fr2,w
	subwf	fr1,w
	btfss	STATUS,2
	endm

; Compare register skip if not equal to literal value.
csne MACRO	fr1,Lit
	movlw	Lit
	subwf	fr1,w
	btfsc	STATUS,2
	endm

; Compare two registers skip if not equal.
csnef MACRO	fr1,fr2
	movf	fr2,w
	subwf	fr1,w
	btfsc	STATUS,2
	endm

; -------------------------------------------------------------------------
; Logical Functions.
; -------------------------------------------------------------------------
; AND register with a literal value.
and	MACRO	fr1,Lit
	movlw	Lit
	andwf	fr1,f
	endm

; AND register fr1 with value in register fr2.
andf MACRO	fr1,fr2
	movf	fr2,w
	andwf	fr1,f
	endm

; OR registers fr1 with a literal value.
or	MACRO	fr1,Lit
	movlw	Lit
	iorwf	fr1,f
	endm

; OR register fr1 with value in register fr2.
orf	MACRO	fr1,fr2
	movf	fr2,w
	iorwf	fr1,f
	endm

; XOR register fr1 with a literal value.
xor	MACRO	fr1,Lit
	movlw	Lit
	xorwf	fr1,f
	endm

; XOR registers fr1 with value in register fr2.
xorf MACRO	fr1,fr2
	movf	fr2,w
	xorwf	fr1,f
	endm

; XOR bit, found in "Bit Control function".

; -------------------------------------------------------------------------
; Other Functions.
; -------------------------------------------------------------------------
#define clr		clrf
#define jmp		goto
#define	sz		skpz
#define	snz		skpnz
#define	sc		skpc
#define	snc		skpnc
#define skip	goto $+2
#define	test	tstf

; Rotate left fr.
rl MACRO fr1
	rlf		fr1,f
	endm

; Rotate right fr.
rr MACRO fr1
	rrf		fr1,f
	endm

; Test value, affects Z, DC and C.
testw MACRO
	iorlw	0x00
	endm

; Jump to label if C is set.
jc	MACRO	Label
	btfsc	STATUS,0
	goto	Label
	endm

; Jump to label if C is cleared.
jnc	MACRO	Label
	btfss	STATUS,0
	goto	Label
	endm

; Test Z, Jump to label if zero.
jz MACRO	Label
	btfsc	STATUS,2
	goto	Label
	endm

; Test Z, Jump to label if not zero.
jnz MACRO	Label
	btfss	STATUS,2
	goto	Label
	endm

; Increment register jump if not zero.
ijnz MACRO	fr1,Label
	incfsz	fr1,f
	goto	Label
	endm

; Decrement register jump if not zero.
djnz MACRO	fr1,Label
	decfsz	fr1,f
	goto	Label
	endm

; Copy Literal value to register fr1.
mov MACRO	fr1,Lit
	movlw	Lit
	movwf	fr1
	endm

; Copy register fr2 to register fr1.
movx MACRO	fr1,fr2
	movf	fr2,w
	movwf	fr1
	endm

; Exchanges the contents of W with register fr1.
invw MACRO	fr1
	xorwf	fr1,F
	xorwf	fr1,W
	xorwf	fr1,F
	endm

; Exchanges the contents between two registers.
invf MACRO	fr1,fr2
	movf	fr2,W
	xorwf	fr1,F
	xorwf	fr1,W
	xorwf	fr1,F
	movwf	fr2
	endm

; Inverts fr1's Upper Nibble with Lower Nibble.
swap MACRO	fr1
	swapf	fr1,f
	endm

; -------------------------------------------------------------------------
; Bit Control function.
; -------------------------------------------------------------------------
#define	clrb	bcf			; Clear bit.
#define	setb	bsf			; Set bit.
#define	sb		btfss		; Skip next instruction if bit = 1.
#define	snb		btfsc		; Skip next instruction if no bit.

; Add bit "fr2,bit" to register fr1
addb MACRO	fr1,fr2,bit
	btfsc	fr2,bit
	incf	fr1,f
	endm

; Subtract bit "fr2,bit" from register fr1
subb MACRO	fr1,fr2,bit
	btfss	fr2,bit
	decf	fr1,f
	endm

; Jump to label if bit.
jb MACRO	fr1,bit,Label
	btfsc	fr1,bit
	goto	Label
	endm

; Jump to label if no bit.
jnb	MACRO	fr1,bit,Label
	btfss	fr1,bit
	goto	Label
	endm

; Move bit.
movb MACRO	fr1,bit,fr2,bit2
	btfss	fr2,bit2
	bcf		fr1,bit
	btfsc	fr2,bit2
	bsf		fr1,bit
	endm

; Move NOT bit.
movbx MACRO	fr1,bit,fr2,bit2
	btfss	fr2,bit2
	bsf		fr1,bit
	btfsc	fr2,bit2
	bcf		fr1,bit
	endm

; XOR bit, flip bit. It does take 5 words instead of 2, but you don't have
; to worry about the location of the flag.
xorb MACRO	fr1,bit
	btfss	fr1,bit
	goto	$+3
	bcf		fr1,bit
	goto	$+2
	bsf		fr1,bit
	endm

; -------------------------------------------------------------------------
; Other Control function.
; -------------------------------------------------------------------------
; These are the same as "banksel" which alsos uses two instructions even if
; they are not needed.
; Move to Bank0.
bank0 MACRO
	bcf		STATUS,RP0
	bcf		STATUS,RP1
	endm

; Move to Bank1.
bank1 MACRO
	bsf		STATUS,RP0
	bcf		STATUS,RP1
	endm

; Move to Bank2.
bank2 MACRO
	bcf		STATUS,RP0
	bsf		STATUS,RP1
	endm

; Move to Bank3.
bank3 MACRO
	bsf		STATUS,RP0
	bsf		STATUS,RP1
	endm

; =========================================================================
; ********** Begining of Word macros
; =========================================================================
; -------------------------------------------------------------------------
; Mathematics Functions.
; -------------------------------------------------------------------------
; Adds a 2-byte literal value to word.
wadd MACRO fr1H,fr1L,litH,litL
	movlw	litL
	addwf	fr1L,f
	skpnc
	incf	fr1H,f
	movlw	litH
	addwf	fr1H,f
	endm

; Adds two words together and puts the result in wd1.
waddf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2L,w
	addwf	fr1L,f
	skpnc
	incf	fr1H,f
	movf	fr2H,w
	addwf	fr1H,f
	endm

; Substracts a 2-byte literal value from word wd1.
wsub MACRO fr1H,fr1L,litH,litL
	movlw	litL
	subwf	fr1L,f
	skpc
	decf	fr1H,f
	movlw	litH
	subwf	fr1H,f
	endm

; Substracts word wd2 from word wd1, result in wd1.
wsubf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2L,w
	subwf	fr1L,f
	skpc
	decf	fr1H,f
	movf	fr2H,w
	subwf	fr1H,f
	endm

; Increment word.
winc MACRO fr1H,fr1L
	incf	fr1L,f
	skpnz
	incf	fr1H,f
	endm

; Decrement word.
wdec MACRO fr1H,fr1L
	decf	fr1L,f
	incf	fr1L,w
	skpnz
	decf	fr1H,f
	endm

; Divide word wd1(fr1H/L) by 2-byte literal value, result goes to RsltH/L,
; remainder ends up in wd1.
wdiv MACRO RsltH,RsltL,fr1H,fr1L,litH,litL
 if ((litH + litL) == 0)
  error " MG: You can not divide by zero! "
 endif
 if (litH  == 0) & (litL  == 1)
  error " MG: No point in dividing by 1! "
 endif
 LOCAL loop,exit
	clrf	RsltH
	clrf	RsltL
loop
	tstf	fr1H			; Exit if word is or just reached zero.
	skpz					; wtest	fr1H,fr1L
	goto	$+2
	tstf	fr1L
	skpnz
	goto	exit
	movlw	litH ^ 0xFF		; wcsae	fr1H,fr1L,litH,litL
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+9
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+4
	movlw	litL
	subwf	fr1L,w
	btfss	STATUS,0
	goto	exit
	movlw	litL			; Subtract one count from dividend.
	subwf	fr1L,f			; wsub	fr1H,fr1L,litH,litL 
	skpc
	decf	fr1H,f
	movlw	litH
	subwf	fr1H,f
	incf	RsltL,f			; Add one to result for every subtraction.
	skpnz					; winc	RsltH,RsltL
	incf	RsltH,f
	goto	loop			; Do until wd1 is less than lit or zero.
exit
	endm

; Divide words fr1 by fr2, result goes to RsltH/L, remainder in fr1.
; If fr1 is less than fr2, the result will be zero and fr1 will remain.
; value in fr1 will change to remainder.     
wdivf MACRO RsltH,RsltL,fr1H,fr1L,fr2H,fr2L
 LOCAL loop,exit
	clrf	RsltH
	clrf	RsltL
loop
	tstf	fr2H
	skpz
	goto	$+2
	tstf	fr2L
	skpnz
	goto	exit
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	exit
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+5
	movf	fr2L,w
	subwf	fr1L,w
	btfss	STATUS,0
	goto	exit
	movf	fr2L,w
	subwf	fr1L,f
	skpc
	decf	fr1H,f
	movf	fr2H,w
	subwf	fr1H,f
	incf	RsltL,f
	skpnz
	incf	RsltH,f
	goto	loop
exit
	endm

; Multiplies fr1 x W, result goes to Result_H/L.
; Value in fr1 will be zero, W will be unchanged.
; Overflow is not possible, maximun result is 0xFE01(65,025).
; This has the same result as macro "mult" in parallax macros.
wmult MACRO	Result_H,Result_L,fr1
 LOCAL loop,exit
	clrf	Result_L
	clrf	Result_H
loop
	movf	fr1,f
	skpnz
	goto	$+6
	addwf	Result_L,f
	skpnc
	incf	Result_H,f
	decf	fr1,f
	goto	loop
	endm

; -------------------------------------------------------------------------
; Comparason Functions.
; -------------------------------------------------------------------------
; Compare word jump if above 2-byte literal value.
wcja MACRO fr1H,fr1L,litH,litL,Label
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+5
	movlw	litL ^ 0xFF
	addwf	fr1L,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare two words jump if wd1 is above wd2.
wcjaf MACRO fr1H,fr1L,fr2H,fr2L,Label
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+5
	movf	fr1L,w
	subwf	fr2L,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare word jump if above or equal to 2-byte literal value.
wcjae MACRO fr1H,fr1L,litH,litL,Label
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+5
	movlw	litL
	subwf	fr1L,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare two words jump if wd1 is above or equal to wd2.
wcjaef MACRO fr1H,fr1L,fr2H,fr2L,Label
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+5
	movf	fr2L,w
	subwf	fr1L,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare word jump if below 2-byte literal value.
wcjb MACRO fr1H,fr1L,litH,litL,Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	Label
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+5
	movlw	litL
	subwf	fr1L,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare two words jump if wd1 is below wd2.
wcjbf MACRO	fr1H,fr1L,fr2H,fr2L,Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	Label
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+5
	movf	fr2L,w
	subwf	fr1L,w
	btfss	STATUS,0
	goto	Label
	endm


; Compare word jump if below or equal to 2-byte literal value.
wcjbe MACRO fr1H,fr1L,litH,litL,Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	Label
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+5
	movlw	litL ^ 0xFF
	addwf	fr1L,w
	btfss	STATUS,0
	goto	Label
	endm

; Compare two words jump if wd1 is below or equal to wd2.
wcjbef MACRO fr1H,fr1L,fr2H,fr2L,Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	Label
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+5
	movf	fr1L,w
	subwf	fr2L,w
	btfsc	STATUS,0
	goto	Label
	endm

; Compare word jump if equal to 2-byte literal value.
wcje MACRO fr1H,fr1L,litH,litL,Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+5
	movlw	litL
	subwf	fr1L,w
	btfsc	STATUS,2
	goto	Label
	endm

; Compare two words jump if equal.
wcjef MACRO	fr1H,fr1L,fr2H,fr2L,Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+5
	movf	fr2L,w
	subwf	fr1L,w
	btfsc	STATUS,2
	goto	Label
	endm

; Compare word jump if not equal to 2-byte literal value.
wcjne MACRO	fr1H,fr1L,litH,litL,Label
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,2
	goto	Label
	movlw	litL
	subwf	fr1L,w
	btfss	STATUS,2
	goto	Label
	endm

; Compare two words and jump if not equal.
wcjnef MACRO fr1H,fr1L,fr2H,fr2L,Label
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,2
	goto	Label
	movf	fr2L,w
	subwf	fr1L,w
	btfss	STATUS,2
	goto	Label
	endm

; -------------------------------------------------------------------------
; Compare word skip if above 2-byte literal value.
wcsa MACRO fr1H,fr1L,litH,litL
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+9				; skip
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+4				; no skip
	movlw	litL ^ 0xFF
	addwf	fr1L,w
	btfss	STATUS,0
	endm

; Compare two wordss skip if wd1 is above wd2.
wcsaf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+9
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+4
	movf	fr1L,w
	subwf	fr2L,w
	btfsc	STATUS,0
	endm

; Compare word skip if above or equal 2-byte literal value.
wcsae MACRO fr1H,fr1L,litH,litL
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+9
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+4
	movlw	litL
	subwf	fr1L,w
	btfss	STATUS,0
	endm

; Compare two words skip if wd1 is above or equal to wd2.
wcsaef MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+9
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+4
	movf	fr2L,w
	subwf	fr1L,w
	btfss	STATUS,0
	endm

; Compare word skip if below the 2-byte literal value.
wcsb MACRO fr1H,fr1L,litH,litL
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+9
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+4
	movlw	litL
	subwf	fr1L,w
	btfsc	STATUS,0
	endm

; Compare two words skip if wd1 is below wd2.
wcsbf MACRO	fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+9
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+4
	movf	fr2L,w
	subwf	fr1L,w
	btfsc	STATUS,0
	endm

; Compare word skip if below or equal 2-byte literal value.
wcsbe MACRO	fr1H,fr1L,litH,litL
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+9
	movlw	litH ^ 0xFF
	addwf	fr1H,w
	btfsc	STATUS,0
	goto	$+4
	movlw	litL ^ 0xFF
	addwf	fr1L,w
	btfsc	STATUS,0
	endm

; Compare two words skip if wd1 is below or equal to wd2.
wcsbef MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,0
	goto	$+9
	movf	fr1H,w
	subwf	fr2H,w
	btfss	STATUS,0
	goto	$+4
	movf	fr1L,w
	subwf	fr2L,w
	btfss	STATUS,0
	endm

; Compare word skip if equal to 2-byte literal value.
wcse MACRO fr1H,fr1L,litH,litL
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+4
	movlw	litL
	subwf	fr1L,w
	btfss	STATUS,2
	endm

; Compare two words skip if equal.
wcsef MACRO	fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+4
	movf	fr2L,w
	subwf	fr1L,w
	btfss	STATUS,2
	endm

; Compare word skip if not equal to 2-byte literal value.
wcsne MACRO	fr1H,fr1L,litH,litL
	movlw	litH
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+5
	movlw	litL
	subwf	fr1L,w
	btfsc	STATUS,2
	endm

; Compare two words skip if not equal.
wcsnef MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	subwf	fr1H,w
	btfss	STATUS,2
	goto	$+5
	movf	fr2L,w
	subwf	fr1L,w
	btfsc	STATUS,2
	endm

; -------------------------------------------------------------------------
; Logical Functions.
; -------------------------------------------------------------------------
; AND word with a 2-byte literal value.
wand MACRO fr1H,fr1L,litH,litL
	movlw	litH
	andwf	fr1H,f
	movlw	litL
	andwf	fr1L,f
	endm

; AND word wd1 with value in word wd2.
wandf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	andwf	fr1H,f
	movf	fr2L,w
	andwf	fr1L,f
	endm

; OR word wd1 with a 2-byte literal value.
wor	MACRO fr1H,fr1L,litH,litL
	movlw	litH
	iorwf	fr1H,f
	movlw	litL
	iorwf	fr1L,f
	endm

; OR word wd1 with value in word wd2.
worf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	iorwf	fr1H,f
	movf	fr2L,w
	iorwf	fr1L,f
	endm

; XOR word wd1 with a 2-byte literal value.
wxor MACRO fr1H,fr1L,litH,litL
	movlw	litH
	xorwf	fr1H,f
	movlw	litL
	xorwf	fr1L,f
	endm

; XOR word wd1 with value in word wd2.
wxorf MACRO fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	xorwf	fr1H,f
	movf	fr2L,w
	xorwf	fr1L,f
	endm

; -------------------------------------------------------------------------
; Other Functions.
; -------------------------------------------------------------------------
; Clears the contents in word.
wclr MACRO	fr1H,fr1L
	clrf	fr1H
	clrf	fr1L
	endm

; Check word Z status.
wtest MACRO	fr1H,fr1L
	tstf	fr1H
	skpz
	goto	$+2
	tstf	fr1L
	endm

; Rotate left wd1.
wrl MACRO fr1H,fr1L
	rlf		fr1L,f
	rlf		fr1H,f
	endm

; Rotate right wd1.
wrr MACRO fr1H,fr1L
	rrf		fr1H,f
	rrf		fr1L,f
	endm

; Decrement word jump if not zero.
wdjnz MACRO fr1H,fr1L,Label
	decf	fr1L,f
	skpz
	goto	$+4
	tstf	fr1H
	skpnz
	goto	$+5
	incf	fr1L,w
	skpnz
	decf	fr1H,f
	goto	Label
	endm

; Increment word jump if not zero.
wijnz MACRO fr1H,fr1L,Label
	incf	fr1L,f	
	skpz
	goto	$+2
	incfsz	fr1H,f
	goto	Label
	endm

; Copy 2-byte Literal value to word wd1.
wmov MACRO	fr1H,fr1L,litH,litL
	movlw	litH
	movwf	fr1H
	movlw	litL
	movwf	fr1L
	endm

; Copy word wd2 to word wd1.
wmovx MACRO	fr1H,fr1L,fr2H,fr2L
	movf	fr2H,w
	movwf	fr1H
	movf	fr2L,w
	movwf	fr1L
	endm

; Swap the contents between H and L bytes of word wd1.
wswap MACRO	fr1H,fr1L
	movf	fr1H,w
	xorwf	fr1L,f
	xorwf	fr1L,w
	xorwf	fr1L,f
	movwf	fr1H
	endm

; -------------------------------------------------------------------------
;
; -------------------------------------------------------------------------
